/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2012. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["dojo/_base/lang"], function(lang){
    lang.getObject("com.ibm.mobile.appcenter.InstallerNative", true);
    
    function getInstalledApplications() {
        var appsArrayStr = window.localStorage.getItem("com.ibm.mobile.appcenter.Installer_appsArray");
        
        if(!appsArrayStr)
            return [];
        else 
            return JSON.parse(appsArrayStr);
    }

    function getInProgressApplications() {
        var appsArrayStr = window.localStorage.getItem("com.ibm.mobile.appcenter.InstallerIP_appsArray");
        
        if(!appsArrayStr)
            return [];
        else 
            return JSON.parse(appsArrayStr);
    }

    function setInstalledApplications(pAppsArray) {
        if(!pAppsArray)
            pAppsArray = [];
        
        window.localStorage.setItem("com.ibm.mobile.appcenter.Installer_appsArray", JSON.stringify(pAppsArray));    
    }
    
    function setInProgressApplications(pAppsArray) {
        if(!pAppsArray)
            pAppsArray = [];
        
        window.localStorage.setItem("com.ibm.mobile.appcenter.InstallerIP_appsArray", JSON.stringify(pAppsArray));
    }
    
    
    function addToList(pAppsArray, pApp) {
        if(!pAppsArray)
            pAppsArray = [];
        
        for(var i = 0 ; i < pAppsArray.length ; i++) {
            if(pAppsArray[i].pkg == pApp.pkg && pAppsArray[i].version == pApp.version) {
                pAppsArray[i] == pApp;
                
                return pAppsArray;
            }
        }
        
        pAppsArray.push(pApp);
        return pAppsArray;
    }

    function removeFromList(pAppsArray, pApp){
        if(!pAppsArray)
            pAppsArray = [];
        
        var rc = [];
            
        for(var i = 0 ; i < pAppsArray.length ; i++) {
            if(pAppsArray[i].pkg == pApp.pkg && pAppsArray[i].version == pApp.version) {
                continue;
            }
            rc.push(pAppsArray[i]);
        }
        
        return rc;
    }
    
    function _alertError(msg) {
        /** @ignore */
        console.log("Error occured: " + msg);
        alert(msg);
    }

    function _printList(pList)
    {
        var outString = "[";
        
        if(pList) {
            for(var i = 0 ; i < pList.length ; i++) {
                var entryStr = "{";
          
                if(pList[i] && pList[i].json) {
                    var entry = JSON.parse(pList[i].json);
                    entryStr += ("package: " + entry.pkg);
                    entryStr += (", version: " + entry.version);
                    entryStr += (", task_uid: " + (entry.task_uid ? entry.task_uid  : ""));
                }
                
                entryStr += "}";
            
                outString += entryStr;
            }
        }
        outString += "]";
        
        return outString;
    }

    com.ibm.mobile.appcenter.InstallerNative = {

        available: false,
            
        /**
         * The secure unique device ID (if registered, else undefined/null)
         * @type string
         */
        _deviceId: "",
                
        _filter: "",

        _initCompleteCallback: null,
        
        _iconUpdateCallback: null,
        
        _applicationsUpdateCallback: null,
        
        _inprogressUpdateCallback: null,
        
        _cancelUpdateCallback: null,
        
        _feedbackRequestCallback: null,
        
        nlsBundle: null,
        
        init: function(pNlsbundle,
        			   initCompleteCallback, 
                       iconUpdateCallback, 
                       applicationsUpdateCallback,
                       inprogressUpdateCallback,
                       cancelUpdateCallback,
                       feedbackRequestCallback,
                       deviceID) {

            if(!deviceID)
                deviceID = 'gals';
            
            console.log("com.ibm.mobile.appcenter.InstallerNative.init - [deviceID =]" + deviceID);

            this.nlsBundle = pNlsbundle;
            this._deviceId                      = deviceID;
            this._initCompleteCallback          = initCompleteCallback;
            this._iconUpdateCallback            = iconUpdateCallback;
            this._applicationsUpdateCallback    = applicationsUpdateCallback;
            this._inprogressUpdateCallback      = inprogressUpdateCallback;
            this._cancelUpdateCallback          = cancelUpdateCallback;
            this._feedbackRequestCallback       = feedbackRequestCallback;

        
            var info = {
                    action: "INIT_COMPLETE",
                    deviceId: this._deviceId,
                    applicationList: this.getInstalledApplications(),
                    iconList: {},
                    inprogressList: this.getInprogressApplications(),
                    width: window.screen.width,
                    height: window.screen.height,
                    filter: ""
            };
            
            var me = this;
            setTimeout(function () {
                
                me._initCompleteCallback(info);
                
            }, 0);                                    
        },            
        
        /**
         * Get the unique secure ID of this device
         * @ignore 
         */
        getDeviceId: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getDeviceId");

            return this._deviceId;
        },


        /**
         * Get the list of applications installed on this device
         * @ignore 
         */
        getInstalledApplications: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getInstalledApplications");

            var apps = [];
            var appArray = getInstalledApplications();
            
            for(var i = 0 ; i < appArray.length ; i++) {
                apps.push({'json': JSON.stringify(appArray[i])});
            }
            
            return apps;
        },


        /**
         * Get the list of cached icons 
         * @ignore 
         */
        getIconsCache: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getIconsCache");

            return { };
        },

        /**
         * Get the list of in progress applications
         * @ignore 
         */
        getInprogressApplications: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getInprogressApplications");

            var apps = [];
            var appArray = getInProgressApplications();
            
            for(var i = 0 ; i < appArray.length ; i++) {
                apps.push({'json': JSON.stringify(appArray[i])});
            }
            
            return apps;
        },

        /**
         * Download and Install application
         * @ignore 
         */
        installApplication: function(pApp) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.installApplication");
            
            if(!pApp || !pApp.task_uid || !pApp.pkg || !pApp.version) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.installApplication - failed inputs");
                return false;
            }
            
            var appArray = addToList(getInProgressApplications(), pApp);
            setInProgressApplications(appArray);
            
            
            this._applicationsUpdateCallback(this.getInstalledApplications(), this.getInprogressApplications());
            
            var me = this;
            setTimeout(function () {
                var ipApps = removeFromList(getInProgressApplications(), pApp);
                setInProgressApplications(ipApps);

                var appArray = addToList(getInstalledApplications(), pApp);
                setInstalledApplications(appArray);
                
                me._applicationsUpdateCallback(me.getInstalledApplications(), me.getInprogressApplications());
                
            }, 10000);                        
            return true;                                                                        
        },

        /**
         * Uninstall an application
         * @ignore 
         */
        removeApplication: function(pApp) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.removeApplication");
            
            if(!pApp || !pApp.task_uid || !pApp.pkg || !pApp.version) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.removeApplication - failed inputs ");
                return false;
            }
            
            var me = this;
            setTimeout(function () {
                var ipApps = removeFromList(getInProgressApplications(), pApp);
                setInProgressApplications(ipApps);
                
                var appArray = removeFromList(getInstalledApplications(), pApp);
                setInstalledApplications(appArray);
                
                me._applicationsUpdateCallback(me.getInstalledApplications(), me.getInprogressApplications());
                
            }, 1000);                        
            return true;                                                                        
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        cancelApplication: function(pApp) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.cancelApplication");
            
            if(!pApp || !pApp.task_uid) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.cancelApplication - failed inputs");
                return false;
            }
            var me = this;
            setTimeout(function () {
                
                var appArray = removeFromList(getInProgressApplications(), pApp);
                setInProgressApplications(appArray);
                
                me._applicationsUpdateCallback(me.getInstalledApplications(), me.getInprogressApplications());
                
            }, 1000);                        
            return true;                                                                        
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        downloadIcon: function(pURL) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.downloadIcon from " + pURL);
            
            return true;
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        updateConnection: function(pUsername, pPassword, pServerURL, pSuccessCallback) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.updateConnection User [" + pUsername + "] Password [" + pPassword + "] Server [" + pServerURL + "]");
            
            if(!pServerURL || !pUsername || !pPassword) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.updateConnection from - failed inputs");
                return false;
            }
            
            var xhttp = new XMLHttpRequest();
            xhttp.open("POST", "http://demo:demo@localhost:9080/applicationcenter/service/device?t=" + new Date().getTime(), false);
            xhttp.setRequestHeader("Content-Type", "application/json");
            xhttp.send("{ device_id : '" + this._deviceId + "', maker : 'Gal', model : 'iGal', os : 'Android' }");
            
            if(xhttp.status != 200)
            {
                alert (xhttp.status + ": " + xhttp.statusText);
            }
            
            setTimeout(pSuccessCallback, 1000);                        
            
            
            return true;            
        },

        /**
         * Update the list of installed applications according to the server's list
         * pApps: JSON array of JSON objects describing the list of installed applications from server. 
         * @ignore 
         */
        syncInstalledApplications: function(pApps) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.syncInstalledApplications" + JSON.stringify(pApps));

            setInstalledApplications(pApps);
            
            return true;                                                                        
        },


        /**
         * TO BE REMOVED LATER
         * 
         */
        hasDoneButton: function() {
            return false;
        },


        getWidth: function() {
            return window.screen.width;      
        },

        getHeight: function() {
            return window.screen.height;
        },

        getFilter: function() {
            return this._filter;
        },

        exitInstaller: function() {
            return;           
        }            
    };
    
    navigator.installer = com.ibm.mobile.appcenter.InstallerNative;
    
    return com.ibm.mobile.appcenter.InstallerNative;
});